{-
This is the module that handles the GLUT procedures needed displaying a bitmap onto the screen
-}

module Extra.MartoGraphics (drawbuffer)
    where

import Graphics.UI.GLUT hiding (Bitmap) 
import Extra.Types
import Settings

-- Transform exact "Pixel number" coordinates to OpenGL "relative" coordinates --
toRelx :: Int -> Float
toRelx x = 2*(fromIntegral x)/wi - 1
 where wi = fromIntegral window_width

toRely :: Int -> Float
toRely y = 2*(hi-(fromIntegral y))/hi - 1
 where hi = fromIntegral window_height

-- The drawing procedures --
drawbuffer :: Bitmap -> IO () -- Initianlization
drawbuffer c = do
  initialWindowSize  $= Size (fromIntegral window_width) (fromIntegral window_height)
  getArgsAndInitialize
  w <- createWindow "Martin Marinov"
  displayCallback $= draw c
  reshapeCallback $= Just (\x -> (viewport $= (Position 0 0, x)))
  draw c
  mainLoop
 
draw :: Bitmap -> IO ()
draw (Bmp _ h c) = do clear [ColorBuffer]
                      render h c
                      flush

render :: Int -> [[Tcolor]] -> IO ()
render _ [] = return ()
render y (d:dat) = putrow 0 y d >> render (y-1) dat

putrow :: Int -> Int -> [Tcolor] -> IO ()
putrow _ _ [] = return ()
putrow x y (c:cs) = putpixel x y c >> putrow (x+1) y cs


color3 :: Float -> Float -> Float -> Color3 Float
color3 = Color3

putpixel :: Int -> Int -> Tcolor -> IO ()
putpixel x y (r, g, b) = let c = color3 ((fromIntegral r)/255) ((fromIntegral g)/255) ((fromIntegral b)/255)
                         in do color c
                               renderPrimitive Points (vertex (Vertex2 (toRelx x) (toRely y)))
